#include <avr/io.h>    // for the input/output registers
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>

#define CPU_FREQ        16000000L       // Assume a CPU frequency of 16Mhz


int valeurPrecedenteBoutons;
int valeurActuelleBoutons; 
int valeurBPStop;
bool vibration;

void init_serial(int speed)
{
  /* Set baud rate */
  UBRR0 = CPU_FREQ/(((unsigned long int)speed)<<4)-1;

  /* Enable transmitter & receiver */
  UCSR0B = (1<<TXEN0 | 1<<RXEN0 | 1<<RXCIE0);

  /* Set 8 bits character and 1 stop bit */
  UCSR0C = (1<<UCSZ01 | 1<<UCSZ00);

  /* Set off UART baud doubler */
  UCSR0A &= ~(1 << U2X0);
  SREG = 0x80;
}


//On paramètre les entrées/sorties.
void IO_init()
{
  DDRD = 0x11001101;   // PIN 1,4,5 as input; PIN 0,2,3,6,7 as output     //PORTD : L7-L6-/-/-L5-L3-Tx-Rx
  DDRB = 0x00010111;  // PIN 3,5,6,7 as input; PIN 0,1,2,4 as output      //PORTB : XTal-XTal-BPStop-V1-MOSI-L10-L9-L8
  DDRC = 0x01110000; // PIN 0,1,2,3,6 as input; PIN 4,5,6 as output       //PORTC : /-Reset-L2-L1-BP1-BP4-BP3-BP2
}


void setup()
{
  IO_init();
  init_serial(9600);
  
  //Ancienne valeur du portC pour les 4 boutons.
  valeurPrecedenteBoutons = (PINC & 0x0F);
  //Valeur actuelle du portC.
  valeurActuelleBoutons = (PINC & 0x0F);
  //Valeur du Bouton Stop.
  valeurBPStop = (PINB & 00100000);
  //Moteurs vibrants.
  vibration = false;

  //Le Bouton BPStop permettant d'arreter l'utilisation de la manette.
  while(valeurBPStop != 00000100)
  {
    check_buttons();
    light_ButtonLeds();
    vibrators();
    
    valeurBPStop = (PINB & 00100000);
  }

  return 0;
}


void sendByte(int valeur)
{
   loop_until_bit_is_set(UCSR0A, UDRE0);
   UDR0 = valeur;
}

void check_buttons ()
{
  //On met à jour la variable d'après le PIND
  valeurActuelleBoutons = (PINC & 0x0F);
  
  //On check si l'état d'un bouton a changé
  if(valeurActuelleBoutons != valeurPrecedenteBoutons)
  {            
    //On envoie le PIND au port série
    sendByte(((valeurPrecedenteBoutons >>2)& 0x3F) + 0x20);
  }

  //On met à jour l'ancienne valeur
  valeurPrecedenteBoutons = valeurActuelleBoutons;
  
  _delay_ms(50);
}

void light_ButtonLeds()
{  
  //On met à jour la variable d'après le PIND
  valeurActuelleBoutons = (PINC & 0x0F);
  
  if(valeurActuelleBoutons != valeurPrecedenteBoutons)
  {
    if (PINC && 0x08 == 0x08){PORTB |= 00000100;} //Bouton 1 : LED10
    else {PORTB &= 11111011;}
      
    if (PINC && 0x01 == 0x01){PORTB |= 00000010;} //Bouton 2 : LED9
    else {PORTB &= 11111101;}
    
    if (PINC && 0x02 == 0x02){PORTB |= 00000001;} //Bouton 3 : LED8
    else {PORTB &= 11111110;}
  
    if (PINC && 0x04 == 0x04){PORTD |= 10000000;} //Bouton 4 : LED7
    else {PORTD &= 01111111;}
  }

  //On met à jour l'ancienne valeur
  valeurPrecedenteBoutons = valeurActuelleBoutons;
}

void vibrators()
{
  if (PINC && 0x0F != 0x00){vibration = true;} //Test sur les boutons.
  else {vibration = false;}
  
  if (vibration == true)
  {
    PORTB |= 00010000; //Vibrator 1 on.
    //Vibrator 2 ne peut être accessible suite à une erreur de montage.
  }
  else 
  {
    PORTB &= 11101111; //Vibrator 1 off.
    //Vibrator 2 ne peut être accessible suite à une erreur de montage.
  }
}

